#define LSIZE 500
#define BUFSIZE 10000
#define MAXRES 1200
#define MAXLINE 150
#define DISLOCATION 8
#define OFFSET 9        /* DISLOCATION+1 */
#define WINSIZ 17       /* 2*DISLOCATION+1 */
#define NPAIRS 136      /* WINSIZ*(WINSIZ-1)/2 */
#define BLANK 21        /* The index of the character '^' in array amino */
#define interpol_coeff 0.75
#define MINFREQ 10.
#define SKIP_BLANK 0
#define Nterm 3
#define Cterm 2
